// service/impl/UserServiceImpl.java
package com.cmeeting.service.impl;

import com.cmeeting.dto.LoginDTO;
import com.cmeeting.mapper.primary.UserMapper;
import com.cmeeting.pojo.User;
import com.cmeeting.service.UserService;
import com.cmeeting.vo.LoginVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.util.DigestUtils;

@Service
public class UserServiceImpl implements UserService {

    @Autowired
    private UserMapper userMapper;

    @Override
    public LoginVO login(LoginDTO loginDTO) {
        // 1. 验证用户名是否存在
        User user = userMapper.findByUsername(loginDTO.getUsername());
        if (user == null) {
            throw new RuntimeException("用户名或密码错误");
        }
        
        // 2. 验证密码
        String inputPassword = encryptPassword(loginDTO.getPassword(), user.getSalt());
        if (!inputPassword.equals(user.getPassword())) {
            throw new RuntimeException("用户名或密码错误");
        }
        
        // 3. 生成token (这里简化处理，实际应该用JWT等)
        String token = generateToken(user);
        
        // 4. 返回登录结果
        LoginVO loginVO = new LoginVO();
        loginVO.setUserId(user.getId());
        loginVO.setUsername(user.getUsername());
        loginVO.setToken(token);
        
        return loginVO;
    }
    
    private String encryptPassword(String password, String salt) {
        // 实际项目中应该使用更安全的加密方式
        return DigestUtils.md5DigestAsHex((password + salt).getBytes());
    }
    
    private String generateToken(User user) {
        // 简化处理，实际应该用JWT等安全方式生成token
        return DigestUtils.md5DigestAsHex((user.getId() + user.getUsername() + System.currentTimeMillis()).getBytes());
    }
}