package com.cmeeting.util;

import io.minio.MinioClient;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.io.ByteArrayInputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

@Component
public class MinioUtils {
    @Value("${minio.endpoint}")
    private String endpoint;
    @Value("${minio.bucketName}")
    private String bucketName;
    @Value("${minio.accessKey}")
    private String accessKey;
    @Value("${minio.secretKey}")
    private String secretKey;

    private MinioClient minioClient;

    public synchronized MinioClient getMinioClient() {
        try {
            if(minioClient == null){
                minioClient = new MinioClient(endpoint, accessKey, secretKey);
            }
            return minioClient;
        }catch (Exception e){
            throw new RuntimeException(e);
        }
    }

    public InputStream getFile(String fileName){
        try{
            getMinioClient();
            InputStream inputStream = minioClient.getObject(bucketName, fileName);
            return inputStream;
        }catch (Exception e){
            throw new RuntimeException(e);
        }
    }

    public InputStream getFile(MinioClient minioClient, String fileName){
        try{
            InputStream inputStream = minioClient.getObject(bucketName, fileName);
            return inputStream;
        }catch (Exception e){
            throw new RuntimeException(e);
        }
    }
    public void upload(String path,InputStream inputStream) throws Exception{
          getMinioClient().putObject(bucketName, path, inputStream, "application/octet-stream" );
    }

    public void upload(String path,byte[] bytes) {
        try (ByteArrayInputStream inputStream = new ByteArrayInputStream(bytes)) {
            // 自动管理流的生命周期，无需手动close
            int data;
            while ((data = inputStream.read()) != -1) {
                System.out.print((char) data);
            }
            upload(path,inputStream);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }

    }

    public Boolean isFileExist(MinioClient minioClient, String fileName){
        boolean found = false;
        try{
            InputStream inputStream = minioClient.getObject(bucketName, fileName);

            found= true;

            return found;

        }catch (Exception e){
            found=false;

            return  false;
        }
    }
}
